/*
 * Copyright (c) 2008, Marcus von Appen
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/**
TODO: implement IAccessibleProxy class, which acts as dummy object with
bindings to the python methods.
*/

#ifndef _PAPI_IACCPROXY_H_
#define _PAPI_IACCPROXY_H_

#include <Python.h>
#include <windows.h>
#include <oleacc.h>

/**
 *
 */
class IAccProxy : public IAccessible
{
public:
    HWND hwnd;
    bool isroot;

    IAccProxy (PyObject *self);
    ~IAccProxy ();

    void getAccessible (VARIANT varChild, IAccProxy **child);
    void ResetPythonRef ();

    /* IAccessible interfaces */
    STDMETHODIMP accDoDefaultAction (VARIANT varChild);
    STDMETHODIMP accHitTest (long xLeft, long yTop, VARIANT *pvarChild);
    STDMETHODIMP accLocation (long *pxLeft, long *pyTop, long *pcxWidth,
        long *pcyHeight, VARIANT varChild);
    STDMETHODIMP accNavigate (long navDir, VARIANT varStart,
        VARIANT *pvarEndUpAt);
    STDMETHODIMP accSelect (long flagsSelect, VARIANT varChild);
    STDMETHODIMP get_accChild (VARIANT varChild, IDispatch **ppdispChild);
    STDMETHODIMP get_accChildCount (long *pcountChildren);
    STDMETHODIMP get_accDefaultAction (VARIANT varChild,
        BSTR *pszDefaultAction);
    STDMETHODIMP get_accDescription (VARIANT varChild, BSTR *pszDescription);
    STDMETHODIMP get_accFocus (VARIANT *pvarChild);
    STDMETHODIMP get_accHelp (VARIANT varChild, BSTR *pszHelp);
    STDMETHODIMP get_accHelpTopic (BSTR *pszHelpFile, VARIANT varChild,
        long *pidTopic);
    STDMETHODIMP get_accKeyboardShortcut (VARIANT varChild,
        BSTR *pszKeyboardShortcut);
    STDMETHODIMP get_accName (VARIANT varChild, BSTR *pszName);
    STDMETHODIMP get_accParent (IDispatch **ppdispParent);
    STDMETHODIMP get_accRole (VARIANT varChild, VARIANT *pvarRole);
    STDMETHODIMP get_accSelection (VARIANT *pvarChildren);
    STDMETHODIMP get_accState (VARIANT varChild, VARIANT *pvarState);
    STDMETHODIMP get_accValue (VARIANT varChild, BSTR *pszValue);
    STDMETHODIMP put_accName (VARIANT varChild, BSTR szName);
    STDMETHODIMP put_accValue (VARIANT varChild, BSTR szValue);

    /* IUnknown interfaces */
    STDMETHODIMP QueryInterface (REFIID riid, void** ppvObject);
    STDMETHODIMP_(ULONG) AddRef ();
    STDMETHODIMP_(ULONG) Release ();

    /* IDispatch interfaes */
    STDMETHODIMP GetTypeInfoCount (unsigned int FAR* pctinfo);
    STDMETHODIMP GetTypeInfo (unsigned int iTInfo, LCID lcid,
        ITypeInfo FAR* FAR* ppTInfo);
    STDMETHODIMP GetIDsOfNames (REFIID riid, OLECHAR FAR* FAR* rgszNames,
        unsigned int cNames, LCID lcid, DISPID FAR* rgDispId);
    STDMETHODIMP Invoke (DISPID dispIdMember, REFIID riid, LCID lcid,
        WORD wFlags, DISPPARAMS FAR* pDispParams,  VARIANT FAR* pVarResult,
        EXCEPINFO FAR* pExcepInfo, unsigned int FAR* puArgErr);

private:
    /* The python object bound to the proxy object to allow a
     * python <-> client interaction.
     */
    PyObject *papiobj;
    ULONG refcnt;
};

#endif /* _PAPI_IACCPROXY_H_ */
