#
# Copyright (c) 2008, Marcus von Appen
# All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  * Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import papi.atk as atk

class ATKAccessible (object):
    """Accessible () -> Accessible
    
    """
    def __init__ (self, acc=None):
        if acc:
            self._acc = acc
        else:
            self._bind_internals ()
        
    def _bind_internals (self, ifaces=atk.ATK_IFACE_INVALID):
        self._acc = atk.AtkObject (ifaces)

    def get_name (self):
        """A.get_name () -> str
        
        Gets the name of the Accessible.
        """
        return self._acc._name
    
    def set_name (self, name):
        """A.set_name (name) -> None
        
        Sets the name of the Accessible.
        """
        self._acc._name = name

    def get_description (self):
        """A.get_description () -> str
        
        Gets the description of the Accessible.
        """
        return self._acc.description

    def set_description (self, description):
        """A.set_description (description) -> None
        
        Sets the description of the Accessible.
        """
        self._acc.description = description
        
    def get_parent (self):
        """A.get_parent () -> str
        
        Gets the parent of the Accessible.
        """
        return self._acc.parent

    def set_parent (self, parent):
        """A.set_parent (parent) -> None
        
        Sets the parent of the Accessible.
        """
        self._acc.parent = parent
        
    def get_child (self, id):
        """A.get_child (id) -> Accessible
        
        Gets the child with the matching id from the Accessible.
        """
        return self._acc.ref_accessible_child (id)
    
    def get_childcount (self):
        """A.get_childcount () -> int
        
        Gets the number of children for the Accessible.
        """
        return self._acc.n_accessible_children

    def get_role (self):
        """A.get_role () -> int
        
        Gets the role of the Accessible.
        """
        return self._acc.role

    def set_role (self, role):
        """A.set_role (role) -> None
        
        Sets the role of the Accessible.
        """
        self._acc.role = role
