import sys
import pygame2
import pygame2.sdl.video as video
import pygame2.sdl.event as event
import pygame2.sdl.constants as const
import papi.atk, atexit

# A simple AtkTextIface implementation.
class SimpleText (papi.atk.AtkObject):
    def __init__ (self):

        # Create out AtkObject, but also let it implement the AtkAction
        # interface.
        papi.atk.AtkObject.__init__ (self,
            papi.atk.ATK_IFACE_EDITABLE_TEXT | papi.atk.ATK_IFACE_TEXT)

        self._text = "Hello World"
        self._offset = 0

        self.ref_state_set = self._ref_state_set
        
        # Override the necessary interfaces of the AtkText.
        self.text_add_selection = self._add_selection
        self.text_get_bounded_ranges = self._get_bounded_ranges
        self.text_get_caret_offset = self._caret_offset
        self.text_get_character_at_offset = self._char_offset
        self.text_get_character_count = self._char_count
        self.text_get_character_extents = self._extents
        self.text_get_default_attributes = self._def_attrs
        self.text_get_n_selections = self._get_selections
        self.text_get_offset_at_point = self._off_at_point
        self.text_get_range_extents = self._range_extents
        self.text_get_run_attributes = self._run_attrs
        self.text_get_selection = self._get_selection
        self.text_get_text = self._get_text
        self.text_get_text_after_offset = self._text_after_offset
        self.text_get_text_at_offset = self._text_at
        self.text_get_text_before_offset = self._text_before
        self.text_remove_selection = self._remove_selection
        self.text_set_caret_offset = self._set_caret
        self.text_set_selection = self._set_selection

        self.editable_set_text_contents = self._set_contents
        self.editable_insert_text = self._insert_text
        self.editable_copy_text = self._copy_text
        self.editable_cut_text = self._cut_text
        self.editable_delete_text = self._delete_text
        self.editable_paste_text = self._paste_text
        self.editable_set_run_attributes = self._set_runattrs

    def _ref_state_set (self):
        return papi.atk.AtkStateSet ()

    def _set_contents (self, text):
        print "Content set to: ", text
        self._text = text
        
    def _insert_text (self, text, pos, len):
        return 0

    def _copy_text (self, start, end):
        return self._text[start:end]

    def _paste_text (self, pos):
        pass

    def _cut_text (self, start, end):
        pass

    def _delete_text (self, start, end):
        pass

    def _set_runattrs (self, attrs, start, end):
        return False

    def _add_selection (self, start, end):
        return False
    
    def _get_bounded_ranges (self, rect, coords, xclip, yclip):
        return None

    def _caret_offset (self):
        print "_caret_offset called"
        return self._offset

    def _char_offset (self, offset):
        return 65

    def _char_count (self):
        print "_char_count called"
        return len (self._text)

    def _extents (self, offset, coords):
        return 3, 3, 10, 10

    def _def_attrs (self):
        return (papi.atk.AtkAttribute ("color", "red"), )

    def _get_selections (self):
        return 0

    def _off_at_point (self, x, y, coords):
        return 0

    def _range_extents (self, start, end, coords):
        return papi.atk.AtkTextRectangle (0, 0, 10, 10)

    def _run_attrs (self, i):
        print "_run_attrs called"
        return (papi.atk.AtkAttribute ("color", "red"), )

    def _get_selection (self, i):
        return "", 0, 0
    
    def _get_text (self, start, end):
        print "_get_text called"
        return self._text[start:end]
    
    def _text_after_offset (self, offset, boundtype):
        return self._text, 0, len (self._text)

    def _text_at (self, offset, boundtype):
        return self._text, 0, len (self._text)

    def _text_before (self, offset, boundtype):
        return self._text, 0, len (self._text)

    def _remove_selection (self, num):
        return False

    def _set_caret (self, offset):
        self._caret = offset
        return True

    def _set_selection (self, num, start, end):
        return False

def _ref_state_set ():
    return papi.atk.AtkStateSet ()

# Create the main application
application = papi.atk.AtkObject ()
application.name = "AtkText Application"
application.description = "An AtkText demonstration example."
application.role = papi.atk.ATK_ROLE_APPLICATION
application.ref_state_set = _ref_state_set

# Set the applicaiton object as root, register the shutdown method, so
# all gets cleaned up nicely and start the ATK system.
papi.atk.set_root (application)
atexit.register (papi.atk.shutdown)
papi.atk.init ()

# Create an action object and denote it as a button in the object
# hierarchy.
action = SimpleText ()
action.name = "Simple AtkText"
action.description = "A simple AtkText."
action.role = papi.atk.ATK_ROLE_TEXT

# Set the parent of the action, so that it can be placed correctly
# within the object hierarchy.
action.parent = application

video.init ()
screen = video.set_mode (320, 240)

print "Press CTRL-C to exit the application."
while True:
    events = event.get ()
    for ev in events:
        if ev.type == const.KEYDOWN:
            if ev.key == const.K_ESCAPE:
                sys.exit ()
            action.editable_set_text_contents (action._text + ev.unicode)
    # Deal with the events by letting the ATK-System process them.
    papi.atk.iterate ()
    
